@@ -46,6 +46,8 @@ from .mtropics import ecoli_houay_pano
 from .mtropics import ecoli_mekong_2016
 from .mtropics import ecoli_mekong
 from .datasets import Quadica
+from .datasets import RRAlpileCatchments
+from .datasets import RRLuleaSweden
 
 
 def busan_beach(inputs: list = None,
@@ -58,7 +60,7 @@ def busan_beach(inputs: list = None,
     The data is in the form of
     mutlivariate time series and was collected over the period of 2 years during
     several precipitation events. The frequency of environmental data is 30 mins
-    while the ARG is discontinuous. The data and its pre-processing is described
+    while that of ARG is discontinuous. The data and its pre-processing is described
     in detail in Jang_ et al., 2021
 
     Arguments
