@@ -1,4 +1,3 @@
-
 import json
 import math
 from typing import Union, Tuple, List
@@ -10,17 +9,17 @@ from ai4water.hyperopt import HyperOpt
 from ai4water.preprocessing import DataSet
 from ai4water.utils.utils import jsonize, ERROR_LABELS
 from ai4water.postprocessing import ProcessPredictions
-from ai4water.utils.utils import clear_weights, dateandtime_now, dict_to_file
+from ai4water.utils.utils import find_best_weight, dateandtime_now, dict_to_file
 
 plot = easy_mpl.plot
 bar_chart = easy_mpl.bar_chart
 taylor_plot = easy_mpl.taylor_plot
 dumbbell_plot = easy_mpl.dumbbell_plot
 
-
 if tf is not None:
     if 230 <= int(''.join(tf.__version__.split('.')[0:2]).ljust(3, '0')) < 250:
         from ai4water.functional import Model
+
         print(f"""
         Switching to functional API due to tensorflow version {tf.__version__} 
         for experiments""")
@@ -40,29 +39,30 @@ SEP = os.sep
 # in order to unify the use of metrics
 Metrics = {
     'regression': lambda t, p, multiclass=False, **kwargs: RegressionMetrics(t, p, **kwargs),
-    'classification': lambda t, p, multiclass=False, **kwargs: ClassificationMetrics(t, p, 
-        multiclass=multiclass, **kwargs)
+    'classification': lambda t, p, multiclass=False, **kwargs: ClassificationMetrics(t, p,
+                                                                                     multiclass=multiclass, **kwargs)
 }
 
 Monitor = {
     'regression': ['r2', 'corr_coeff', 'mse', 'rmse', 'r2_score',
-                'nse', 'kge', 'mape', 'pbias', 'bias', 'mae', 'nrmse',
-                'mase'],
+                   'nse', 'kge', 'mape', 'pbias', 'bias', 'mae', 'nrmse',
+                   'mase'],
 
     'classification': ['accuracy', 'precision', 'recall', 'mse']
 }
 
+
 class Experiments(object):
     """
     Base class for all the experiments.
 
     All the expriments must be subclasses of this class.
-    The core idea of ``Experiments`` is based upon ``model``. An experiment 
-    consists of one or more models. The models differ from each other in their 
-    structure/idea/concept/configuration. When :py:meth:`ai4water.experiments.Experiments.fit` 
-    is called, each ``model`` is built and trained. The user can customize, building 
-    and training process by subclassing this class and customizing 
-    :py:meth:`ai4water.experiments.Experiments._build` and 
+    The core idea of ``Experiments`` is based upon ``model``. An experiment
+    consists of one or more models. The models differ from each other in their
+    structure/idea/concept/configuration. When :py:meth:`ai4water.experiments.Experiments.fit`
+    is called, each ``model`` is built and trained. The user can customize, building
+    and training process by subclassing this class and customizing
+    :py:meth:`ai4water.experiments.Experiments._build` and
     :py:meth:`ai4water.experiments.Experiments._fit` methods.
 
     Attributes
@@ -86,6 +86,7 @@ class Experiments(object):
     - fit_with_tpot
 
     """
+
     def __init__(
             self,
             cases: dict = None,
@@ -125,7 +126,7 @@ class Experiments(object):
 
         if cases is None:
             cases = {}
-        self.cases = {'model_'+key if not key.startswith('model_') else key: val for key, val in cases.items()}
+        self.cases = {'model_' + key if not key.startswith('model_') else key: val for key, val in cases.items()}
         self.models = self.models + list(self.cases.keys())
 
         self.exp_path = os.path.join(os.getcwd(), "results", self.exp_name)
@@ -133,7 +134,7 @@ class Experiments(object):
             os.makedirs(self.exp_path)
 
         self.update_config(models=self.models, exp_path=self.exp_path,
-            exp_name=self.exp_name, cases=self.cases)
+                           exp_name=self.exp_name, cases=self.cases)
 
         if monitor is None:
             self.monitor = Monitor[self.mode]
@@ -186,12 +187,21 @@ class Experiments(object):
 
         return
 
+    def _named_x0(self)->dict:
+        x0 = getattr(self, 'x0', None)
+        param_space = getattr(self, 'param_space', None)
+        if param_space:
+            names = [s.name for s in param_space]
+            if x0:
+                return {k: v for k, v in zip(names, x0)}
+        return {}
+
     def fit(
             self,
             x=None,
             y=None,
             data=None,
-            validation_data:tuple = None,
+            validation_data: tuple = None,
             run_type: str = "dry_run",
             opt_method: str = "bayes",
             num_iterations: int = 12,
@@ -341,32 +351,32 @@ class Experiments(object):
                         title=f"{self.exp_name}{SEP}{model_name}",
                         **config)
 
+                # there may be attributes int the model, which needs to be loaded so run the method first.
+                # such as param_space etc.
+                if hasattr(self, model_type):
+                    getattr(self, model_type)()
+
                 if run_type == 'dry_run':
                     if self.verbosity >= 0: print(f"running  {model_type} model")
-                    train_results, test_results = objective_fn()
+                    train_results, test_results = objective_fn(**self._named_x0())
                     self._populate_results(model_name, train_results, test_results)
                 else:
-                    # there may be attributes int the model, which needs to be loaded so run the method first.
-                    # such as param_space etc.
-                    if hasattr(self, model_type):
-                        getattr(self, model_type)()
-
                     opt_dir = os.path.join(os.getcwd(),
-                        f"results{SEP}{self.exp_name}{SEP}{model_name}")
+                                           f"results{SEP}{self.exp_name}{SEP}{model_name}")
 
-                    if self.verbosity > 0: 
+                    if self.verbosity > 1:
                         print(f"optimizing  {model_type} using {opt_method} method")
 
                     self.optimizer = HyperOpt(
                         opt_method,
                         objective_fn=objective_fn,
-                        param_space=self.param_space, 
+                        param_space=self.param_space,
                         opt_path=opt_dir,
                         num_iterations=num_iterations,  # number of iterations
                         x0=self.x0,
                         verbosity=self.verbosity,
                         **hpo_kws
-                        )
+                    )
 
                     self.opt_results = self.optimizer.fit()
 
@@ -406,22 +416,23 @@ class Experiments(object):
             **kwargs
     ):
         """Evaluate the best models."""
-        best_models = clear_weights(opt_dir, rename=False, write=False)
-        # TODO for ML, best_models is empty
-        if len(best_models) < 1:
+
+        folders = [path for path in os.listdir(opt_dir) if os.path.isdir(os.path.join(opt_dir, path)) and path.startswith('1_')]
+        # TODO for ML, best_models can be empty
+        if len(folders) < 1:
             return self.train_best(x, y, data, validation_data, model_type)
-        for mod, props in best_models.items():
-            mod_path = os.path.join(props['path'], "config.json")
-            mod_weights = props['weights']
+
+        for mod_path in folders:
+            config_path = os.path.join(opt_dir, mod_path, "config.json")
+            best_weights = find_best_weight(os.path.join(opt_dir, mod_path, "weights"))
 
             train_results, test_results = self.build_from_config(
                 data,
-                mod_path, 
-                mod_weights, 
+                config_path,
+                best_weights,
                 **kwargs)
 
-            if mod.startswith('1_'):
-                self._populate_results(model_type, train_results, test_results)
+            self._populate_results(model_type, train_results, test_results)
         return
 
     def train_best(
@@ -446,7 +457,7 @@ class Experiments(object):
             predict=True,
             model={_model: self.optimizer.best_paras()},
             title=f"{self.exp_name}{SEP}{model_type}{SEP}best"
-            )
+        )
 
         self._populate_results(model_type, train_results, test_results)
         return
@@ -464,7 +475,7 @@ class Experiments(object):
             'train': {
                 'true': {
                     'std': np.std(train_results[0])},
-                'simulation':{
+                'simulation': {
                     'std': np.std(train_results[1])}},
 
             'test': {
@@ -476,6 +487,8 @@ class Experiments(object):
         # save performance metrics of train and test
         metrics = Metrics[self.mode](train_results[0],
                                      train_results[1],
+                                     replace_nan=True,
+                                     replace_inf=True,
                                      multiclass=self.model_.is_multiclass)
         train_metrics = {}
         for metric in self.monitor:
@@ -483,6 +496,8 @@ class Experiments(object):
 
         metrics = Metrics[self.mode](test_results[0],
                                      test_results[1],
+                                     replace_nan=True,
+                                     replace_inf=True,
                                      multiclass=self.model_.is_multiclass)
         test_metrics = {}
         for metric in self.monitor:
@@ -498,7 +513,7 @@ class Experiments(object):
             exclude: Union[None, list] = None,
             figsize: tuple = (9, 7),
             **kwargs
-    )->plt.Figure:
+    ) -> plt.Figure:
         """
         Compares the models using taylor_plot_.
 
@@ -613,8 +628,8 @@ Available cases are {self.models} and you wanted to include
             metric_name: str,
             plot_type: str = 'dumbell',
             save: bool = True,
-            name : str = '',
-            dpi : int = 200,
+            name: str = '',
+            dpi: int = 200,
             **kwargs
     ) -> pd.DataFrame:
         """
@@ -694,8 +709,8 @@ Available cases are {self.models} and you wanted to include
             for ordr in order:
                 bar_chart(improvement[ordr], improvement.index.tolist(),
                           ax=ax, color=colors[ordr], show=False,
-                          xlabel=ERROR_LABELS.get(metric_name, metric_name),
-                          label=ordr, **kwargs)
+                          ax_kws={'xlabel':ERROR_LABELS.get(metric_name, metric_name),
+                          'label':ordr}, **kwargs)
 
             ax.legend()
             plt.title('Improvement after Optimization')
@@ -788,26 +803,29 @@ Available cases are {self.models} and you wanted to include
         fig.set_figheight(kwargs.get('fig_height', 8))
         fig.set_figwidth(kwargs.get('fig_width', 8))
 
+        labels = [model.split('model_')[1] for model in models.index.tolist()]
+        models.index = labels
+
         bar_chart(ax=axis[0],
                   labels=models.index.tolist(),
                   values=models['train'],
                   color=kwargs.get('color', None),
-                  title="Train",
-                  xlabel=ERROR_LABELS.get(matric_name, matric_name),
-                  xlabel_fs=kwargs.get('xlabel_fs', 16),
-                  title_fs=kwargs.get('title_fs', 20),
+                  ax_kws={'title':"Train",
+                  'xlabel':ERROR_LABELS.get(matric_name, matric_name),
+                  'xlabel_kws':{'fontsize': kwargs.get('xlabel_fs', 16)},
+                  'title_kws':{'fontsize': kwargs.get('title_fs', 20)}},
                   show=False,
                   )
 
         bar_chart(ax=axis[1],
                   labels=models.index.tolist(),
                   values=models['test'],
-                  title="Test",
                   color=kwargs.get('color', None),
-                  xlabel=ERROR_LABELS.get(matric_name, matric_name),
-                  xlabel_fs=kwargs.get('xlabel_fs', 16),
-                  title_fs=kwargs.get('title_fs', 20),
-                  show_yaxis=False,
+                  ax_kws={'title':"Test",
+                  'xlabel':ERROR_LABELS.get(matric_name, matric_name),
+                  'xlabel_kws':{'fontsize': kwargs.get('xlabel_fs', 16)},
+                  'title_kws':{'fontsize': kwargs.get('title_fs', 20)},
+                  'show_yaxis':False},
                   show=False
                   )
 
@@ -850,7 +868,7 @@ Available cases are {self.models} and you wanted to include
             figsize : tuple
                 size of the figure
             **kwargs :
-                any other keyword arguments to be passed to the 
+                any other keyword arguments to be passed to the
                 `plot <https://easy-mpl.readthedocs.io/en/latest/plots.html#easy_mpl.plot>`_
         Returns
         -------
@@ -1139,10 +1157,10 @@ Available cases are {self.models} and you wanted to include
 
         if cutoff_type is not None:
             assert cutoff_val is not None
-            if cutoff_type=="greater":
-                df = df.loc[df[sort_by]>cutoff_val]
+            if cutoff_type == "greater":
+                df = df.loc[df[sort_by] > cutoff_val]
             else:
-                df = df.loc[df[sort_by]<cutoff_val]
+                df = df.loc[df[sort_by] < cutoff_val]
 
         return df
 
@@ -1226,7 +1244,7 @@ Available cases are {self.models} and you wanted to include
 
         if isinstance(models, int):
 
-            assert len(self.metrics)>1, f"""
+            assert len(self.metrics) > 1, f"""
             you must first run .fit() method in order to choose top {models} models"""
 
             # sort the models w.r.t their performance
@@ -1272,7 +1290,7 @@ Available cases are {self.models} and you wanted to include
             json.dump(jsonize(tpot_config), fp, indent=True)
 
         tpot = tpot_caller(
-            verbosity=self.verbosity+1,
+            verbosity=self.verbosity + 1,
             scoring=scoring,
             config_dict=tpot_config,
             **tpot_args
@@ -1294,11 +1312,10 @@ Available cases are {self.models} and you wanted to include
         else:
             mode = "classification"
         visualizer = ProcessPredictions(path=self.exp_path,
-                                        show=self.verbosity,
+                                        show=bool(self.verbosity),
                                         mode=mode)
 
         for idx, data_name in enumerate(['training', 'test']):
-
             x_data, y_data = getattr(dh, f"{data_name}_data")(key=str(idx))
 
             pred = tpot.fitted_pipeline_.predict(x_data)
@@ -1306,8 +1323,8 @@ Available cases are {self.models} and you wanted to include
 
             # todo, perform inverse transform and deindexification
             visualizer(
-                pd.DataFrame(y_data.reshape(-1,)),
-                pd.DataFrame(pred.reshape(-1,)),
+                pd.DataFrame(y_data.reshape(-1, )),
+                pd.DataFrame(pred.reshape(-1, )),
             )
         # save the python code of fitted pipeline
         tpot.export(os.path.join(self.exp_path, "tpot_fitted_pipeline.py"))
@@ -1365,7 +1382,7 @@ Available cases are {self.models} and you wanted to include
 
         suggested_paras = jsonize(suggested_paras)
 
-        verbosity = max(self.verbosity-1, 0)
+        verbosity = max(self.verbosity - 1, 0)
         if 'verbosity' in self.model_kws:
             verbosity = self.model_kws.pop('verbosity')
 
@@ -1392,7 +1409,7 @@ Available cases are {self.models} and you wanted to include
         if cross_validate:
             if validation_data is None:
                 return self.model_.cross_val_score(data=data,
-                    scoring = self.model_.config['val_metric'])
+                                                   scoring=self.model_.config['val_metric'])
             else:
                 return self.model._cross_val_score(
                     x=x, y=y  # todo, combine validation data here with x,y
@@ -1404,11 +1421,11 @@ Available cases are {self.models} and you wanted to include
             self,
             validation_data=None,
             data="validation"
-    )->float:
+    ) -> float:
         """Evaluates the model"""
 
         if validation_data is None:
-            t, p = self.model_.predict(
+            t, p = self.model_.predict_on_validation_data(
                 data=data,
                 return_true=True,
                 process_results=False)
@@ -1420,8 +1437,11 @@ Available cases are {self.models} and you wanted to include
             )
 
         metrics = Metrics[self.mode](t, p,
-            remove_zero=True, remove_neg=True,
-            multiclass=self.model_.is_multiclass)
+                                     remove_zero=True,
+                                     remove_neg=True,
+                                     replace_nan=True,
+                                     replace_inf=True,
+                                     multiclass=self.model_.is_multiclass)
 
         test_metrics = {}
         for metric in self.monitor:
@@ -1434,7 +1454,7 @@ Available cases are {self.models} and you wanted to include
         if self.model_.config['val_metric'] in [
             'r2', 'nse', 'kge', 'r2_mod', 'r2_adj', 'r2_score', 'accuracy', 'f1_score']:
             val_score = 1.0 - val_score
-        
+
         if not math.isfinite(val_score):
             val_score = 9999  # TODO, find a better way to handle this
 
@@ -1454,7 +1474,7 @@ Available cases are {self.models} and you wanted to include
         if validation_data is None:
             test_true, test_pred = self.model_.predict(data=data, return_true=True)
 
-            train_true, train_pred = self.model_.predict(data='training', return_true=True)
+            train_true, train_pred = self.model_.predict_on_training_data(data=data, return_true=True)
         else:
             test_true, test_pred = self.model_.predict(*validation_data, return_true=True)
             train_true, train_pred = self.model_.predict(x=x, y=y, return_true=True)
@@ -1510,7 +1530,7 @@ class TransformationExperiments(Experiments):
                  x0=None,
                  cases: dict = None,
                  exp_name: str = None,
-                 num_samples: int = 5, 
+                 num_samples: int = 5,
                  verbosity: int = 1,
                  **model_kws):
         self.param_space = param_space
@@ -1540,7 +1560,7 @@ be used to build ai4water's Model class.
 
         suggested_paras = jsonize(suggested_paras)
 
-        verbosity = max(self.verbosity-1, 0)
+        verbosity = max(self.verbosity - 1, 0)
         if 'verbosity' in self.model_kws:
             verbosity = self.model_kws.pop('verbosity')
 
@@ -1555,8 +1575,8 @@ be used to build ai4water's Model class.
         return
 
     def build_from_config(self, data, config_path, weight_file, **kwargs):
-
         model = Model.from_config_file(config_path=config_path)
+        assert weight_file is not None, f"{config_path}"
         weight_file = os.path.join(model.w_path, weight_file)
         model.update_weights(weight_file=weight_file)
 
@@ -1586,10 +1606,9 @@ def sort_array(array):
     return [np.min(results[:i]) for i in iters]
 
 
-def consider_exclude(exclude: Union[str, list], 
-        models, 
-        models_to_filter: Union[list, dict] = None):
-
+def consider_exclude(exclude: Union[str, list],
+                     models,
+                     models_to_filter: Union[list, dict] = None):
     if isinstance(exclude, str):
         exclude = [exclude]
 
@@ -1611,14 +1630,12 @@ def consider_exclude(exclude: Union[str, list],
 
 
 def load_json_file(fpath):
-
     with open(fpath, 'r') as fp:
         result = json.load(fp)
     return result
 
 
 def save_json_file(fpath, obj):
-
     with open(fpath, 'w') as fp:
         json.dump(jsonize(obj), fp, sort_keys=True, indent=4)
 
@@ -1629,15 +1646,13 @@ def shred_model_name(model_name):
     return key
 
 
-
 def verify_data(
         x=None,
         y=None,
         data=None,
         validation_data=None,
         label="validation"
-)->Tuple[dict, dict]:
-
+) -> Tuple[dict, dict]:
     def num_examples(samples):
         if isinstance(samples, list):
             assert len(set(len(sample) for sample in samples)) == 1
@@ -1668,8 +1683,8 @@ def verify_data(
         assert isinstance(data, pd.DataFrame), f"data must be dataframe, but it is {type(data)}"
 
     if validation_data is not None:
-
-        assert isinstance(validation_data, (tuple, list)), f"{label} data must be of type tuple but it is {type(validation_data)}"
+        assert isinstance(validation_data,
+                          (tuple, list)), f"{label} data must be of type tuple but it is {type(validation_data)}"
         assert len(validation_data) == 2, f"{label}_data tuple must have length 2 but it has {len(validation_data)}"
         assert isinstance(validation_data[1], np.ndarray), f"second value in {label}_data must be ndarray"
         assert num_examples(validation_data[0]) == num_examples(validation_data[1])
