@@ -2,13 +2,11 @@ import json
 import copy
 import inspect
 import warnings
-from typing import Union
+from typing import Union, Dict
 from collections import OrderedDict
 
-from ai4water.utils.utils import JsonEncoder
 from .utils import plot_convergences
 from .utils import get_one_tpe_x_iter
-from ai4water.utils.utils import Jsonize, dateandtime_now
 from .utils import to_skopt_as_dict
 from .utils import post_process_skopt_results
 from .utils import to_skopt_space
@@ -16,7 +14,11 @@ from .utils import save_skopt_results
 from ._space import Categorical, Real, Integer
 from .utils import sort_x_iters, x_iter_for_tpe
 from .utils import loss_histogram, plot_hyperparameters
-from ai4water.utils.visualizations import plot_edf
+
+from ai4water.utils.utils import JsonEncoder
+from ai4water.utils.utils import clear_weights
+from ai4water.utils.utils import Jsonize, dateandtime_now
+from ai4water.utils.visualizations import edf_plot
 from ai4water.backend import hyperopt as _hyperopt
 from ai4water.backend import np, pd, plt, os, sklearn, optuna, plotly, skopt, easy_mpl
 
@@ -302,7 +304,7 @@ class HyperOpt(object):
         Parameters
         ----------
             algorithm : str
-                must be one of "random", "grid" "bayes", "bayes_rf", and "tpe", defining which
+                must be one of ``random``, ``grid``, ``bayes``, ``bayes_rf``, and ``tpe``, defining which
                 optimization algorithm to use.
             objective_fn : callable
                 Any callable function whose returned value is to be minimized.
@@ -345,7 +347,7 @@ class HyperOpt(object):
         self.param_space = param_space
         self.original_space = param_space       # todo self.space and self.param_space should be combined.
         self.title = self.algorithm
-        self.results = {}  # internally stored results
+        self.results = OrderedDict()  # internally stored results
         self.gpmin_results = None  #
         self.data = None
         self.eval_on_best = eval_on_best
@@ -620,9 +622,8 @@ Backend must be one of hyperopt, optuna or sklearn but is is {x}"""
     def best_paras(self, as_list=False) -> Union[list, dict]:
         # returns best parameters either as dictionary or as list
         if self.use_skopt_gpmin:
-            d = self.xy_of_iterations()
-            k = list(dict(sorted(d.items())).keys())[0]
-            paras = d[k]
+            xys = self.xy_of_iterations()
+            paras = xys[self.best_iter()]['x']
         elif self.backend == 'hyperopt':
             d = get_one_tpe_x_iter(self.trials.best_trial['misc']['vals'], self.hp_space())
             if as_list:
@@ -636,8 +637,7 @@ Backend must be one of hyperopt, optuna or sklearn but is is {x}"""
         elif self.use_skopt_bayes or self.use_sklearn:
             paras = self.optfn.best_params_
         else:
-            best_y = list(sorted(self.results.keys()))[0]
-            paras = sort_x_iters(self.results[best_y], list(self.param_space.keys()))
+            paras = sort_x_iters(self.results[self.best_iter()]['x'], list(self.param_space.keys()))
 
         if as_list:
             return list(paras.values())
@@ -753,7 +753,8 @@ Backend must be one of hyperopt, optuna or sklearn but is is {x}"""
         if len(self.results) < 1:
             fv = search_result.func_vals
             xiters = search_result.x_iters
-            self.results = {f'{round(k, 8)}_{idx}': self.to_kw(v) for idx, k, v in zip(range(self.num_iterations), fv, xiters)}
+            for idx, y, x in zip(range(len(fv)), fv, xiters):
+                self.results[idx] = {'y': y, 'x': x}
 
         if self._process_results:
             post_process_skopt_results(search_result, self.results, self.opt_path)
@@ -786,9 +787,10 @@ Backend must be one of hyperopt, optuna or sklearn but is is {x}"""
                         this argument is set to True during initiatiation of HyperOpt.""")
             err = round(err, 8)
 
-            self.results[err + idx] = sort_x_iters(para, self.original_para_order())
+            self.results[idx] = {'y':err, 'x':sort_x_iters(para, self.original_para_order())}
 
         if self._process_results:
+            clear_weights(self.opt_path, self.results)
             self.process_results()
 
         if self.eval_on_best:
@@ -914,37 +916,48 @@ Backend must be one of hyperopt, optuna or sklearn but is is {x}"""
         """returns a dictionary whose values are hyperopt equivalent space instances."""
         return {k: v.as_hp(False if self.algorithm == 'atpe' else True) for k,v in self.space().items()}
 
-    def xy_of_iterations(self) -> dict:
-        # todo, not in original order
+    def xy_of_iterations(self) -> Dict[int,Dict[str, Union[str, dict]]]:
+        """returns a dictionary whose keys are iteration numbers are values are xy parirs
+        at those iterations.
+
+        Returns
+        Dict[int, Dict[str, [dict,float]]]
+        """
         if self.backend == "optuna":
             num_iters = range(self.num_iterations)
-            return {float(f'{round(trial.value, 5)}_{idx}'): trial.params for idx, trial in zip(num_iters, self.study.trials)}
+            results = {}
+            for idx, trial in zip(num_iters, self.study.trials):
+                results[idx] = {'y': trial.value, 'x': trial.params}
+            return results
         elif self.backend == "hyperopt":
             return x_iter_for_tpe(self.trials, self.hp_space(), as_list=False)
         elif self.backend == 'skopt':
             assert self.gpmin_results is not None, f"gpmin_results is not populated yet"
-            # adding idx because sometimes the difference between two func_vals is negligible
             fv = self.gpmin_results['func_vals']
             xiters = self.gpmin_results['x_iters']
-            return {f'{round(k, 5)}_{idx}': self.to_kw(v) for idx, k,v in zip(range(len(fv)), fv, xiters)}
+            results = {}
+            for idx, y, x in zip(range(len(fv)), fv, xiters):
+                results[idx] = {'y': y, 'x': self.to_kw(x)}
+            return results
         else:
             # for sklearn based
             return self.results
 
-    def func_vals(self):
+    def func_vals(self)->np.ndarray:
+        """returns the value of objective function at each iteration."""
         if self.backend == 'hyperopt':
-            return [self.trials.results[i]['loss'] for i in range(self.num_iterations)]
+            return np.array([self.trials.results[i]['loss'] for i in range(self.num_iterations)])
         elif self.backend == 'optuna':
-            return [s.values for s in self.study.trials]
+            return np.array([s.values for s in self.study.trials])
         else:
-            return np.array(list(self.results.keys()), dtype=np.float32)
+            return np.array([v['y'] for v in self.results.values()])
 
     def skopt_results(self):
         if self.use_own and self.algorithm in ["bayes", "bayes_rf"] and self.backend == 'skopt':
             return self.gpmin_results
         else:
             class SR:
-                x_iters = [list(s.values()) for s in self.xy_of_iterations().values()]
+                x_iters = [list(s['x'].values()) for s in self.xy_of_iterations().values()]
                 func_vals = self.func_vals()
                 space = self.skopt_space()
                 if isinstance(self.best_paras(), list):
@@ -956,32 +969,37 @@ Backend must be one of hyperopt, optuna or sklearn but is is {x}"""
 
             return SR()
 
+    def best_iter(self)->int:
+        """returns the iteration on which best/optimized parameters are obtained.
+        The indexing starts from 0.
+        """
+        return np.nanargmin(self.func_vals()).item()
+
     def best_xy(self) -> dict:
         """Returns best (optimized) parameters as dictionary.
-        The key of the dictionary is the value of the objective function with
-        best parameters. The value of the dictionary is again a dictionay of
-        optimized parameters.
+        The dictionary has two keys ``x`` and ``y``. ``x`` is the
+        best hyperparameters while `y` is the corresponding objective function value.
         """
         d = self.xy_of_iterations()
-        k = list(dict(sorted(d.items())).keys())[0]
-        paras = {k: d[k]}
+        key = list(d.keys())[self.best_iter()]
 
-        return paras
+        return d[key]
 
-    def _plot_edf(self, save=True):
-        # empirical CDF of objective function
+    def _plot_edf(self, save=True, show=False, **kwargs):
+        """empirical CDF of objective function"""
         plt.close("all")
         y = np.array(list(self.xy_of_iterations().keys())).astype("float64")
-        plot_edf(y)
+        edf_plot(y, show=show, **kwargs)
         if save:
             plt.savefig(os.path.join(self.opt_path, "edf"))
         return
 
     def _plot_parallel_coords(self, save=True, **kwargs):
-        # parallel coordinates of hyperparameters
+        """ parallel coordinates of hyperparameters
+        """
         d = self.xy_of_iterations()
 
-        data = pd.DataFrame([list(v.values()) for v in d.values()],
+        data = pd.DataFrame([list(v['x'].values()) for v in d.values()],
                             columns=[s for s in self.space()])
         categories = np.array(list(self.xy_of_iterations().keys())).astype("float64")
         parallel_coordinates(
@@ -1004,10 +1022,14 @@ Backend must be one of hyperopt, optuna or sklearn but is is {x}"""
                         bbox_inches='tight')
         return
 
-    def _plot_convergence(self, save=True):
+    def _plot_convergence(self, original=False, save=True, **kwargs):
         plt.close('all')
-        # todo, should include an option to plot original evaluations instead of only minimum
-        plot_convergence([self.skopt_results()])
+        if original:
+            easy_mpl.plot(self.func_vals(), '--.',
+                 xlabel="Number of calls $n$",
+                 ylabel=r"$\min f(x)$ after $n$ calls", **kwargs)
+        else:
+            plot_convergence([self.skopt_results()], **kwargs)
         if save:
             fname = os.path.join(self.opt_path, "convergence.png")
             plt.savefig(fname, dpi=300, bbox_inches='tight')
@@ -1104,12 +1126,14 @@ Backend must be one of hyperopt, optuna or sklearn but is is {x}"""
             from pandas.plotting._matplotlib.tools import _subplots as create_subplots
 
         # name of hyperparameters
-        h_paras = list(list(self.best_xy().values())[0].keys())
+        h_paras = list(self.best_xy()['x'].keys())
 
         # container with a list for each hyperparameter
         h_para_lists = {k: [] for k in h_paras}
 
-        for score, x_iter in self.xy_of_iterations().items():
+        for xy in self.xy_of_iterations().values():
+            #score = xy['y']
+            x_iter = xy['x']
             for para, val in x_iter.items():
                 h_para_lists[para].append(val)
 
@@ -1262,8 +1286,8 @@ Backend must be one of hyperopt, optuna or sklearn but is is {x}"""
 
             distributions = {sn: s.to_optuna() for sn, s in self.space().items()}
 
-            for _y, _x in self.xy_of_iterations().items():
-
+            for xy in self.xy_of_iterations().values():
+                _x, _y = xy['x'], xy['y']
                 assert isinstance(_x, dict), f'params must of type dict but provided params are of type {_x.__class__.__name__}'
 
                 trials.append(_Trial(number=idx,
