@@ -1,6 +1,6 @@
+
 import json
 from itertools import islice
-from pickle import PicklingError
 from collections import OrderedDict
 
 try:
@@ -46,10 +46,10 @@ def x_iter_for_tpe(trials, param_space: dict, as_list=True):
         vals = t['misc']['vals']
         y = t['result']['loss']
 
-        _iter = get_one_tpe_x_iter(vals, param_space)
-        iterations[f'{round(y, 5)}_{idx}'] = _iter
+        x = get_one_tpe_x_iter(vals, param_space)
+        iterations[idx] = {'x': x, 'y': y}
 
-        x_iters.append(_iter)
+        x_iters.append(x)
 
     if as_list:
         return [list(d.values()) for d in x_iters]
@@ -295,11 +295,7 @@ def post_process_skopt_results(skopt_results, results, opt_path):
 
     skopt_plots(skopt_results, pref=opt_path)
 
-    if 'folder' in list(results.items())[0]:
-        clear_weights(results=results, opt_dir=opt_path)
-    else:
-        clear_weights(results=results, opt_dir=opt_path)
-        clear_weights(opt_dir=opt_path)
+    clear_weights(results=results, opt_dir=opt_path)
 
     return
 
@@ -310,12 +306,6 @@ def save_skopt_results(skopt_results, opt_path):
 
     sr_res = SerializeSKOptResults(skopt_results)
 
-    # try:  # adding .hpo extension so that no other file/folder exists with same name
-    #     fname = os.path.join(opt_path, os.path.basename(opt_path) + ".hpo")
-    #     dump(skopt_results, fname)
-    # except PicklingError:
-    #     print("could not pickle results")
-
     try:
         with open(fname + '.json', 'w') as fp:
             json.dump(sr_res.serialized_results, fp, sort_keys=True, indent=4)
