@@ -28,4 +28,7 @@ from ._transformations import Log10Scaler
 from ._transformations import TanScaler
 from ._transformations import CumsumScaler
 from ._transformations import Center
-from ._transformations import FunctionTransformer
\ No newline at end of file
+from ._transformations import FunctionTransformer
+from ._transformations import ParetoTransformer
+from ._transformations import VastTransformer
+from ._transformations import MmadTransformer
\ No newline at end of file
